package org.artifactory.security.token;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.access.CreatedTokenInfo;
import org.artifactory.api.security.access.TokenSpec;
import org.artifactory.aql.AqlConverts;
import org.artifactory.aql.AqlService;
import org.artifactory.aql.api.domain.sensitive.AqlApiItem;
import org.artifactory.aql.result.AqlEagerResult;
import org.artifactory.aql.result.rows.AqlItem;
import org.artifactory.descriptor.repo.RepoDescriptor;
import org.artifactory.fs.FileInfo;
import org.artifactory.repo.RepoPath;
import org.artifactory.security.access.AccessService;
import org.artifactory.security.token.model.PathToken;
import org.artifactory.security.token.model.TokenUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.ws.rs.NotFoundException;
import java.util.List;
import java.util.stream.Collectors;

import static org.artifactory.aql.api.domain.sensitive.AqlApiItem.name;
import static org.artifactory.aql.api.domain.sensitive.AqlApiItem.repo;
import static org.artifactory.aql.api.internal.AqlBase.and;
import static org.artifactory.repo.RepoPathFactory.create;
import static org.artifactory.security.token.model.TokenUtils.expiresInOrDefault;
import static org.artifactory.security.token.model.TokenUtils.requireUsername;

/**
 * @author barh
 */
@Service
public class PathTokenServiceImpl implements PathTokenService {

    private RepositoryService repositoryService;
    private AccessService accessService;
    private AqlService aqlService;
    private AuthorizationService authorizationService;

    @Autowired
    public PathTokenServiceImpl(RepositoryService repositoryService, AccessService accessService, AqlService aqlService,
            AuthorizationService authorizationService) {
        this.repositoryService = repositoryService;
        this.accessService = accessService;
        this.aqlService = aqlService;
        this.authorizationService = authorizationService;
    }

    public PathToken createPathToken(String repoKey, String filename, Long expiresIn) {
        String username = authorizationService.currentUsername();
        TokenUtils.requireUsername(username);
        expiresIn = TokenUtils.expiresInOrDefault(expiresIn);
        String serverId = accessService.getArtifactoryServiceId().toString();
        String path = getBlobPath(repoKey, filename);
        if (path == null) {
            throw new NotFoundException();
        }
        return wrappedCreateToken(expiresIn, username, serverId, path);
    }

    public PathToken createPathToken(String filepath, Long expiresIn) {
        String username = authorizationService.currentUsername();
        requireUsername(username);
        expiresIn = expiresInOrDefault(expiresIn);
        String serverId = accessService.getArtifactoryServiceId().toString();
        RepoPath repoPath = create(filepath);
        RepoDescriptor repoDescriptor = getRepoDescriptor(repoPath.getRepoKey());
        if (repoDescriptor == null) {
            throw new NotFoundException();
        }
        String key = repoDescriptor.getKey();
        String path = repoPath.getPath();

        if (!repoDescriptor.isReal()) {
            FileInfo fileInfo = repositoryService.getVirtualFileInfo(repoPath);
            key = fileInfo.getRepoKey();
            path = fileInfo.getRepoPath().getPath();
        }
        String targetPath = String.format("%s/%s", key, path);
        return wrappedCreateToken(expiresIn, username, serverId, targetPath);
    }

    private PathToken wrappedCreateToken(Long expiresIn, String username, String serverId, String targetPath) {
        try {
            return createToken(username, expiresIn, serverId, targetPath);
        } catch (org.springframework.security.acls.model.NotFoundException e) {
            throw new NotFoundException();
        }
    }


    private String getBlobPath(String repoKey, String filename) {
        RepoDescriptor repoDescriptor = getRepoDescriptor(repoKey);
        if (repoDescriptor == null) {
            return null;
        }
        FileInfo blob = findBlob(repoDescriptor.getKey(), filename);
        if (blob == null) {
            return null;
        }
        return blob.getRepoKey() + "/" + blob.getRepoPath().getPath();
    }

    private FileInfo findBlob(String repoKey, String filename) {
        AqlApiItem query = AqlApiItem.create().filter(
                and(
                        repo().equal(repoKey),
                        name().equal(filename)
                )
        ).limit(1);
        return executeQuery(query);
    }

    private FileInfo executeQuery(AqlApiItem query) {
        AqlEagerResult<AqlItem> allBlobs = aqlService.executeQueryEager(query);
        List<FileInfo> fileInfos = allBlobs.getResults().stream()
                .map(AqlConverts.toFileInfo)
                .collect(Collectors.toList());

        return fileInfos.isEmpty() ? null : fileInfos.get(0);
    }


    private RepoDescriptor getRepoDescriptor(String repoKey) {
        RepoDescriptor repoDescriptor = repositoryService.localOrCachedRepoDescriptorByKey(repoKey);
        if (repoDescriptor == null) {
            return repositoryService.virtualRepoDescriptorByKey(repoKey);
        }
        return repoDescriptor;
    }

    private PathToken createToken(String username, Long expiresIn, String serverId, String targetPath) {
        List<String> scope = TokenUtils.createScopeFromPath(targetPath);

        TokenSpec tokenSpec = TokenUtils.createTokenSpec(username)
                .scope(scope)
                .expiresIn(expiresIn)
                .refreshable(false)
                .audience(TokenUtils.audienceToList(serverId));
        TokenUtils.assertSupportedTokenSpecForCreate(tokenSpec,
                accessService.getArtifactoryServiceId().getFormattedName());
        CreatedTokenInfo tokenInfo = accessService.createToken(tokenSpec);

        return new PathToken(targetPath, tokenInfo.getTokenValue());
    }

}
