package org.artifactory.metadata.service;

import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.descriptor.repo.HttpRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RemoteRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.opentracing.ArtifactoryJaegerTracerConfiguration;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.spring.ArtifactoryApplicationContext;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.version.ArtifactoryVersionProvider;
import org.awaitility.Awaitility;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;


/**
 * @author Uriah Levy
 */
@Test
public class MetadataReindexConverterTest extends ArtifactoryHomeBoundTest {
    @Mock
    RepositoryService repositoryService;

    @Mock
    MetadataEventService metadataEventService;

    @Mock
    ArtifactoryApplicationContext applicationContext;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        ArtifactoryContextThreadBinder.bind(applicationContext);
        when(applicationContext.beanForType(MetadataEventService.class)).thenReturn(metadataEventService);
        when(applicationContext.beanForType(RepositoryService.class)).thenReturn(repositoryService);
        when(applicationContext.beanForType(ArtifactoryTracer.class)).thenReturn(new ArtifactoryTracer(ArtifactoryJaegerTracerConfiguration.initTracer()));
    }

    @AfterClass
    public void unbind() {
        ArtifactoryContextThreadBinder.unbind();
    }

    public void testReindexByType() {
        MetadataReindexConverter metadataReindexConverter = new MetadataReindexConverter(Set.of(RepoType.Docker, RepoType.NuGet), true);

        LocalRepoDescriptor dockerLocalDescriptor = new LocalRepoDescriptor();
        dockerLocalDescriptor.setKey("docker-local");
        RemoteRepoDescriptor dockerRemoteDescriptor = new HttpRepoDescriptor();
        dockerRemoteDescriptor.setKey("docker-remote");
        when(repositoryService.getRepoDescriptorByPackageType(RepoType.Docker)).thenReturn(List.of(dockerLocalDescriptor, dockerRemoteDescriptor));

        LocalRepoDescriptor nugetRepoDescriptor = new LocalRepoDescriptor();
        nugetRepoDescriptor.setKey("nuget-local");
        when(repositoryService.getRepoDescriptorByPackageType(RepoType.NuGet)).thenReturn(List.of(nugetRepoDescriptor));

        metadataReindexConverter.convert();

        Awaitility.await().atMost(5, TimeUnit.SECONDS)
                .untilAsserted(() -> verify(metadataEventService).reindexSync(List.of("docker-local"), true));
        Awaitility.await().atMost(5, TimeUnit.SECONDS)
                .untilAsserted(() -> verify(metadataEventService).reindexSync(List.of("docker-remote"), true));
        Awaitility.await().atMost(5, TimeUnit.SECONDS)
                .untilAsserted(() -> verify(metadataEventService).reindexSync(List.of("nuget-local"), true));
    }

    public void testConverterRunCriteria() {
        assertFalse(new MetadataReindexConverter(Set.of(), false).isInterested(ArtifactoryVersionProvider.v6170m001.get()));
        assertTrue(new MetadataReindexConverter(Set.of(), false).isInterested(ArtifactoryVersionProvider.v700m001.get()));
    }
}