package org.artifactory.opentracing;

import io.jaegertracing.internal.JaegerTracer;
import io.opentracing.Scope;
import io.opentracing.Span;
import io.opentracing.SpanContext;
import io.opentracing.propagation.Format;
import io.opentracing.propagation.TextMapAdapter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Nullable;
import java.util.Map;

/**
 * Wraps a tracer impl (current impl used is Jaeger Tracer: https://www.jaegertracing.io).
 * Manages the lifecycle of server-side tracing spans. Use this to create spans, or retrieve information about existing
 * spans (such as the trace-id, etc)
 *
 * @author Uriah Levy
 */
@Component
public class ArtifactoryTracer {

    private static final Logger log = LoggerFactory.getLogger(ArtifactoryTracer.class);
    public static final String UBER_TRACE_ID = "uber-trace-id";
    private static final String COLON = ":";
    private JaegerTracer tracer;

    @Autowired
    public ArtifactoryTracer(JaegerTracer tracer) {
        this.tracer = tracer;
    }

    /**
     * @return the scoped (current thread) span, if an active span exists
     */
    @Nullable
    public ServerTracingSpan getCurrentThreadActiveSpan() {
        if (tracer.activeSpan() != null) {
            return new ServerTracingSpan(tracer.activeSpan());
        }
        return null;
    }

    /**
     * Sets the given {@link Span} as the active span on this thread
     *
     * @param span - the span
     */
    public void setActiveSpan(ServerTracingSpan span) {
        tracer.scopeManager().activate(span.getSpan());
        MDC.put(UBER_TRACE_ID, getCurrentSpanTraceId());
    }

    /**
     * Starts a scoped (current thread) span. It is the callers responsibility to close the returned Scope which will
     * finish the underlying span as well, otherwise the Span will remain active.
     *
     * @return the {@link Scope} that represent the newly activated span
     */
    public ServerTracingScope startSpanOnCurrentThread(String operation) {
        return new ServerTracingScope(tracer.buildSpan(operation).startActive(true), getCurrentSpanTraceId());
    }

    public ServerTracingScope startSpanOnCurrentThread(String operation, Map<String, String> requestHeaders) {
        SpanContext parentSpan = extractSpanContextFromRequest(requestHeaders);
        if (parentSpan != null) {
            log.debug("Found existing Tracing Span. A new child Server Tracing Span will be associated with it");
            return new ServerTracingScope(tracer.buildSpan(operation).asChildOf(parentSpan).startActive(true),
                    getCurrentSpanTraceId());
        }
        return new ServerTracingScope(tracer.buildSpan(operation).startActive(true), getCurrentSpanTraceId());
    }

    /**
     * Get the full context tracing ID in the format of:  {trace-id}:{span-id}:{span-id}:0
     * The span ID repeats since the third element originally belongs to a parent-span-id which is now deprecated
     * see https://www.jaegertracing.io/docs/1.12/client-libraries/#trace-span-identity
     *
     * @return the full tracing context ID
     */
    public String getFullContextTracingId() {
        if (tracer.activeSpan() != null) {
            String spanId = tracer.activeSpan().context().toSpanId();
            String traceId = tracer.activeSpan().context().toTraceId();
            return traceId + COLON + spanId + COLON + spanId + COLON + 0;
        }
        return null;
    }

    public void closeTracer() {
        tracer.close();
    }

    @Nullable
    String getCurrentSpanTraceId() {
        if (tracer.activeSpan() != null) {
            return tracer.activeSpan().context().toTraceId();
        }
        return null;
    }

    /**
     * Extracts {@link SpanContext} from request headers.
     *
     * @return A Span Context extracted from request headers
     */
    @Nullable
    private SpanContext extractSpanContextFromRequest(Map<String, String> requestHeaders) {
        if (requestHeaders != null) {
            return tracer.extract(Format.Builtin.HTTP_HEADERS, new TextMapAdapter(requestHeaders));
        }
        return null;
    }
}
